#!/bin/bash

error() {
	printf "%s\n" "$@" >&2
	exit 1
}

cd "$(dirname "$0")" || error "cd test dir"

# option to regenerate output we compare with
# use with care if podman_start output changed
# or after confirming new tests work
REGENERATE="${REGENERATE:-}"

[ -z "$REGENERATE" ] || rm -f podman_start/*/output.*

check_output() {
	local output="$1"
	shift
	local cmd=( "$@" )

	echo "${cmd[*]}"

	if [ -n "$REGENERATE" ]; then
		"${cmd[@]}" > "$d/output.$output"
	else
		diff -u "$d/output.$output" <("${cmd[@]}") \
			|| error "'${cmd[*]}' produced different output than expected"
	fi

}


# prepare and clean test dir.
# we need a stable path for this as it appears in logs,
# so we can't use mktemp
rm -rf /tmp/podman_start
mkdir /tmp/podman_start || error "Need /tmp/podman_start to be writable"

# verify dry run mode, always
for d in podman_start/*; do
	[ -d "$d" ] || continue
	check_output all ../podman/podman_start \
		--networksdir /tmp/podman_start/networks \
		--linksdir /tmp/podman_start/links \
		-n -c "$d" -a
	for f in "$d/"*.conf; do
		b=$(basename "$f" .conf)
		check_output "$b" ../podman/podman_start \
			--networksdir /tmp/podman_start/networks \
			--linksdir /tmp/podman_start/links \
			-n -c "$d" "$b"
	done
done

# cleanup temp dir
rm -rf /tmp/podman_start

# if podman is available also run some containers
command -v podman > /dev/null || exit 0

# setup test requirements
export TESTDIR=$(mktemp -d /tmp/abos-base.podman.XXXXXX) \
	|| error "Could not create temp dir"
trap "find '$TESTDIR' -xdev -exec sh -c 'for f; do mountpoint -q \"\$f\" && umount \"\$f\"; done' - {} +; rm -rf '$TESTDIR'" EXIT
mkdir "$TESTDIR/test space"
echo foo > "$TESTDIR/test space/foo"
mkdir "$TESTDIR/test"
echo bar > "$TESTDIR/test/bar"
podman image exists docker.io/alpine \
	|| podman pull docker.io/alpine

for d in podman_start/run_*; do
	for f in "$d"/*.conf; do
		b=$(basename "$f" .conf)
		if podman container exists "$b"; then
			podman rm "$b" ||error "Could not cleanup $b, still running?"
		fi
		../podman/podman_start \
				--networksdir /tmp/podman_start/networks \
				--linksdir /tmp/podman_start/links \
				-v -c "$d" "$b" \
			|| error "podman_start -c $d $b failed"
		timeout 10s podman wait "$b" \
			|| error "container $b ($d) didn't stop within timeout?"
		check_output "run_$b" podman logs "$b"
		podman rm "$b"
	done
done


# test podman network initialization.
# This has to run as root, on alpine -- and 'podman network inspect'
# does not work in a container... grmblgrmblgrmbl.
if grep -qx "ID=alpine" /etc/os-release && [ "$(id -u)" = 0 ]; then (
	. ../podman/podman_start --source

	NETWORKSDIR="$TESTDIR/networks"
	rm -rf "$NETWORKSDIR"

	init_network

	podman network inspect podman \
		| sed -e '1d;$d' \
		      -e 's/\(dns_enabled.*\)false/\1true/' \
		      -e 's/^ *//' -e 's/"created": "[^"]*"/"created": ""/' \
		      -e 's/"id": "[^"]*"/"id": ""/' \
		      > "$NETWORKSDIR/podman.json_podman_inspect"
	sed -e 's/^ *//' -e 's/"created": "[^"]*"/"created": ""/' \
	    -e 's/"id": "[^"]*"/"id": ""/' \
		< "$NETWORKSDIR/podman.json" \
		> "$NETWORKSDIR/podman.json_podman_start"

	diff -u "$NETWORKSDIR/podman.json_podman_inspect" \
		"$NETWORKSDIR/podman.json_podman_start" \
		|| error "generated podman network config differs from podman's"

) || error "init_network test failed"
fi

# lastly test mklink_hostpath directly as it requires root and must be skipped
if [ "$(id -u)" = 0 ]; then (
	. ../podman/podman_start --source
	LINKSDIR="$TESTDIR/links"
	NETWORKSDIR="$TESTDIR/networks"
	name=contname
	devnum=0

	# dir
	hostpath="$TESTDIR/test space"
	mklink_hostpath
	[ "$hostpath" = "$LINKSDIR/${name}_0" ] \
		|| error "hostpath $hostpath not correct (dir)"
	[ -e "$hostpath"/foo ] || error "hostpath not mounted"
	podman run -v "$hostpath:/work" --rm docker.io/alpine test -e /work/foo \
		|| error "podman did not handle mount correctly (dir)"

	# file
	hostpath="$TESTDIR/test space/foo"
	mklink_hostpath
	[ "$hostpath" = "$LINKSDIR/${name}_1" ] \
		|| error "hostpath $hostpath not correct (file)"
	[ "$(cat "$hostpath")" = "foo" ] || error "hostpath content not correct"
	[ "$(podman run -v "$hostpath:/work" --rm docker.io/alpine cat /work)" = "foo" ] \
		|| error "podman did not handle mount correctly (file)"

	# remove dir/file
	devnum=0
	hostpath="$TESTDIR/test space/foo"
	mklink_hostpath
	[ "$hostpath" = "$LINKSDIR/${name}_0" ] \
		|| error "hostpath $hostpath not correct (remove dir)"
	hostpath="$TESTDIR/test space/foo"
	mklink_hostpath
	[ "$hostpath" = "$LINKSDIR/${name}_1" ] \
		|| error "hostpath $hostpath not correct (remove file)"
) || error "mklist_hostpath test failed"
fi
